@@ -41,15 +41,29 @@ def dtype_fst2numpy (datyp, nbits=None):
     datyp = 5
   return dtype_fst2numpy(datyp,nbits)
 
+# Indicate if pandas should be used or not.
+_pandas_needed = False
+def _use_pandas ():
+  if not _pandas_needed: return False
+  try:
+    import pandas
+    return True
+  except ImportError:
+    return False
+
 # Decorator for efficiently converting a scalar function to a vectorized
 # function.
 def vectorize (f):
   from functools import wraps
-  try:
-    from pandas import Series, unique
-    import numpy as np
-    @wraps(f)
-    def vectorized_f (x):
+  import numpy as np
+  def cached_f(x, cache={}):
+    if x not in cache:
+      cache[x] = f(x)
+    return cache[x]
+  @wraps(f)
+  def vectorized_f (x):
+    if _use_pandas():
+      from pandas import Series, unique
       # If we're given a scalar value, then simply return it.
       if not hasattr(x,'__len__'):
         return f(x)
@@ -60,13 +74,7 @@ def vectorize (f):
       table = dict(zip(inputs,outputs))
       result = Series(x).map(table)
       return result.values
-  except ImportError:
-    def cached_f(x, cache={}):
-      if x not in cache:
-        cache[x] = f(x)
-      return cache[x]
-    @wraps(f)
-    def vectorized_f (x):
+    else:
       # If we're given a scalar value, then simply return it.
       if not hasattr(x,'__len__'):
         return cached_f(x)
@@ -132,6 +140,28 @@ class _iter_type (_base_type):
     self.record_id = record_id
     self.deps = []
 
+# Similar to above, but more general.
+# Can allow for multiple records along the inner axes.
+class _chunk_type (_base_type):
+  __slots__ = ('name','atts','axes','dtype','chunks','chunksize','deps')
+  def __init__ (self, name, atts, axes, dtype, chunks, chunksize):
+    self.name = name
+    self.atts = atts
+    self.axes = axes
+    self.dtype = dtype
+    self.chunks = chunks
+    self.chunksize = chunksize
+    self.deps = []
+  def items(self):
+    for key, value in self.chunks.items():
+      key = tuple(k if isinstance(k,int) else slice(*k) for k in key)
+      yield key, value
+  def keys(self):
+    for key, value in self.items():
+      yield key
+  def values(self):
+    return self.chunks.values()
+
 # Fake progress bar - does nothing.
 class _FakeBar (object):
   def __init__ (self, *args, **kwargs): pass
@@ -228,18 +258,6 @@ class BufferBase (object):
   # Uses string formatting operations on the variable metadata.
   _human_var_id = ('%(nomvar)s', '%(ni)sx%(nj)s', '%(nk)sL')
 
-  # Field names and types for storing record headers in a structured array.
-  _headers_dtype = [
-     ('file_id','int32'),('key','int32'),
-     ('dateo','int32'),('datev','int32'),('deet','int32'),('npas','int32'),
-     ('ni','int32'),('nj','int32'),('nk','int32'),('nbits','int32'),
-     ('datyp','int32'),('ip1','int32'),('ip2','int32'),('ip3','int32'),
-     ('typvar','|S2'),('nomvar','|S4'),('etiket','|S12'),('grtyp','|S1'),
-     ('ig1','int32'),('ig2','int32'),('ig3','int32'),('ig4','int32'),
-     ('swa','int32'),('lng','int32'),('dltf','int32'),('ubc','int32'),
-     ('xtra1','int32'),('xtra2','int32'),('xtra3','int32'),('ismeta','bool'),
-  ]
-
   # Record parameters which should not be used as nc variable attributes.
   # (They're either internal to the file, or part of the data, not metadata).
   _ignore_atts = ('file_id','swa','lng','dltf','ubc','xtra1','xtra2','xtra3','key','shape','d')
@@ -260,8 +278,7 @@ class BufferBase (object):
     group.add_argument('--rpnstd-metadata-list', metavar='nomvar,...', help=_("Specify a minimal set of RPN record attributes to include in the output file."))
     parser.add_argument('--ignore-typvar', action='store_true', help=_('Tells the converter to ignore the typvar when deciding if two records are part of the same field.  Default is to split the variable on different typvars.'))
     parser.add_argument('--ignore-etiket', action='store_true', help=_('Tells the converter to ignore the etiket when deciding if two records are part of the same field.  Default is to split the variable on different etikets.'))
-    parser.add_argument('--no-quick-scan', action='store_true', help=SUPPRESS)
-    #help=_('Don't read record headers from the raw librmn structures, instead call fstprm.  This can be much slower, but safer.')
+    parser.add_argument('--serial', action='store_true', help=_('Disables multithreading/multiprocessing.  Useful for resource-limited machines.'))
 
   # Do some checks on the command-line arguments after parsing them.
   @classmethod
@@ -294,7 +311,7 @@ class BufferBase (object):
     from rpnpy.librmn.base import fnom
     from rpnpy.librmn.fstd98 import fstouv
     from rpnpy.librmn.const import FST_RO
-    from fstd2nc.extra import librmn
+    from rpnpy.librmn import librmn
     opened_file_id = getattr(self,'_opened_file_id',-1)
     # Check if this file already opened.
     if opened_file_id == file_id:
@@ -325,12 +342,7 @@ class BufferBase (object):
   ###############################################
   # Basic flow for reading data
 
-  # Filter out any Buffer arguments from object.__new__.
-  # Otherwise, get an error when running with Python3.
-  def __new__ (cls, *args, **kwargs):
-    return object.__new__(cls)
-
-  def __init__ (self, filename, header_cache=None, progress=False, minimal_metadata=None, rpnstd_metadata=None, rpnstd_metadata_list=None, ignore_typvar=False, ignore_etiket=False, no_quick_scan=False):
+  def __init__ (self, filename, _headers=None, progress=False, minimal_metadata=None, rpnstd_metadata=None, rpnstd_metadata_list=None, ignore_typvar=False, ignore_etiket=False, serial=False):
     """
     Read raw records from FSTD files, into the buffer.
     Multiple files can be read simultaneously.
@@ -355,15 +367,23 @@ class BufferBase (object):
         Tells the converter to ignore the etiket when deciding if two
         records are part of the same field.  Default is to split the
         variable on different etikets.
+    serial : bool, optional
+        Disables multithreading/multiprocessing.  Useful for resource-limited
+        machines.
     """
     from rpnpy.librmn.fstd98 import fstnbr, fstinl, fstprm, fstopenall
     from rpnpy.librmn.const import FST_RO
-    from fstd2nc.extra import maybeFST as isFST
+    from fstd2nc.extra import raw_headers, decode_headers
     from collections import Counter
     import numpy as np
     from glob import glob, has_magic
     import os
     import warnings
+    from multiprocessing import Pool
+    try:
+      from itertools import imap  # Python 2
+    except ImportError:
+      imap = map
 
     # Set up lock for threading.
     # The same lock is shared for all Buffer objects, to synchronize access to
@@ -397,6 +417,8 @@ class BufferBase (object):
       self._var_id = self._var_id[0:1] + ('etiket',) + self._var_id[1:]
       self._human_var_id = self._human_var_id[0:1] + ('%(etiket)s',) + self._human_var_id[1:]
 
+    self._serial = serial
+
     if isinstance(filename,str):
       infiles = [filename]
     else:
@@ -413,54 +435,46 @@ class BufferBase (object):
         else:
           expanded_infiles.append((infile,f))
 
-    # Inspect all input files, and extract the headers from valid RPN files.
+    # Extract headers from the files.
+    if len(expanded_infiles) == 1: Bar = _FakeBar
+    bar = Bar(_("Inspecting input files"), suffix='%(percent)d%% (%(index)d/%(max)d)', max=len(expanded_infiles))
+
+    if len(expanded_infiles) > 1 and not self._serial:
+      with Pool() as p:
+        headers = p.imap (raw_headers, [f for (infile,f) in expanded_infiles])
+        headers = bar.iter(headers)
+        headers = list(headers) # Start scanning.
+    else:
+      headers = imap (raw_headers, [f for (infile,f) in expanded_infiles])
+      headers = bar.iter(headers)
+      headers = list(headers) # Start scanning.
+
+    bar.finish()
+
+    # Check which files had headers used, report on the results.
     matches = Counter()
-    headers = []
     self._files = []
-    if header_cache is None: header_cache = {}
-
-    # Show a progress bar when there are multiple input files.
-    if len(expanded_infiles) > 1:
-      expanded_infiles = Bar(_("Inspecting input files"), suffix='%(percent)d%% (%(index)d/%(max)d)').iter(expanded_infiles)
-
-    for infile, f in expanded_infiles:
-      fkey = f
-      if fkey.startswith('/'):
-        fkey = '__ROOT__'+fkey
-      if fkey not in header_cache and (not os.path.exists(f) or not isFST(f)):
+    file_ids = []
+    for i, (infile, f) in enumerate(expanded_infiles):
+      if headers[i] is not None:
+        matches[infile] += 1
+        filenum = len(self._files)
+        self._files.append(f)
+        file_ids.extend([filenum]*(len(headers[i])//72))
+      else:
         matches[infile] += 0
-        continue
-      matches[infile] += 1
 
-      # Read the headers from the file(s) and store the info in the table.
-      filenum = len(self._files)
-      self._files.append(f)
-      if fkey not in header_cache:
-        funit = self._open(filenum)
-        nrecs = fstnbr(funit)
-        h = np.zeros(nrecs, dtype=self._headers_dtype)
-
-        if no_quick_scan:
-          keys = fstinl(funit)
-          params = map(fstprm, keys)
-          for i,prm in enumerate(params):
-            for n,v in prm.items():
-              if n in h.dtype.names:
-                h[n][i] = v
-        else:
-          from fstd2nc.extra import all_params
-          params = all_params(funit,out=h)
-          keys = params['key']
-
-        # Encode the keys without the file index info.
-        h['key'] = keys
-        h['key'] >>= 10
-        header_cache[fkey] = h
-      h = header_cache[fkey]
-      # The file info will be an index into a separate file list.
-      h['file_id'] = filenum
-
-      headers.append(h)
+    # Decode all the headers
+    headers = [h for h in headers if h is not None]
+    # Remember indices in the headers (needed for reconstructing keys)
+    indices = [range(len(h)//72) for h in headers]
+    if len(headers) > 0:
+      headers = np.concatenate(headers)
+      indices = np.concatenate(indices)
+      headers = decode_headers(headers)
+      # Encode the keys without the file index info.
+      headers['key'] = (indices % 256) | ((indices//256)<<9)
+      headers['file_id'] = np.array(file_ids, dtype='int32')
 
     # Check if the input entries actually matched anything.
     for infile, count in matches.items():
@@ -476,23 +490,37 @@ class BufferBase (object):
         else:
           warn(_("Problem with input file '%s'")%infile)
 
-    nfiles = len(headers)
+    nfiles = len(self._files)
     if nfiles == 0:
       error(_("no input files found!"))
+    elif nfiles > 10:
+      global _pandas_needed
+      _pandas_needed = True
     info(_("Found %d RPN input file(s)"%nfiles))
 
-    self._headers = np.ma.concatenate(headers)
+    # Add extra headers? (hack for generating Buffer objects from external
+    # sources (not FSTD files)
+    if _headers is not None:
+      headers = dict(_headers)
+      # Keep link to first file for metadata purposes below?
+      headers['file_id'] = np.zeros(len(headers['nomvar']), dtype='int32')
 
+    self._headers = headers
+    self._nrecs = len(headers['nomvar'])
 
     # Find all unique meta (coordinate) records, and link a subset of files
     # that provide all unique metadata records.
     # This will make it easier to look up the meta records later.
-    meta_mask = np.zeros(len(self._headers),dtype='bool')
-    for meta_name in self._meta_records + self._maybe_meta_records:
+    meta_mask = np.zeros(self._nrecs,dtype='bool')
+    for meta_name in self._meta_records:
       meta_name = (meta_name+b'   ')[:4]
       meta_mask |= (self._headers['nomvar'] == meta_name)
+    for meta_name in self._maybe_meta_records:
+      meta_name = (meta_name+b'   ')[:4]
+      meta_mask |= (self._headers['nomvar'] == meta_name) & ((self._headers['ni']==1)|(self._headers['nj']==1))
 
     # Store this metadata identification in case it's useful for some mixins.
+    self._headers['ismeta'] = np.empty(self._nrecs, dtype='bool')
     self._headers['ismeta'][:] = meta_mask
 
     meta_recids = np.where(meta_mask)[0]
@@ -501,7 +529,9 @@ class BufferBase (object):
     # Suppress FutureWarning from numpy about doing this.  Probably benign...
     with warnings.catch_warnings():
       warnings.simplefilter("ignore")
-      meta_keys = self._headers.data[meta_mask][list(self._var_id)+['ip1','ip2','ip3','ig1','ig2','ig3','ig4']]
+      from fstd2nc.extra import structured_array
+      headers = structured_array(self._headers)
+      meta_keys = headers.data[meta_mask][list(self._var_id)+['ip1','ip2','ip3','ig1','ig2','ig3','ig4']]
     meta_keys, ind = np.unique(meta_keys, return_index=True)
     meta_recids = meta_recids[ind]
     # Find the files that give these unique coord records.
@@ -525,13 +555,14 @@ class BufferBase (object):
     from collections import OrderedDict
     import numpy as np
     import warnings
+    from fstd2nc.extra import structured_array
 
-    nrecs = len(self._headers)
+    nrecs = self._nrecs
 
     # Degenerate case: no data in buffer
     if nrecs == 0: return
 
-    records = self._headers
+    records = structured_array(self._headers)
 
     # Ignore deleted / invalidated records, and coordinate records.
     valid = (records['dltf'] == 0) & (records['ismeta'] == 0)
@@ -691,15 +722,15 @@ class BufferBase (object):
     import pandas as pd
     import warnings
 
-    nrecs = len(self._headers)
+    nrecs = self._nrecs
 
     # Degenerate case: no data in buffer
     if nrecs == 0: return
 
     # Convert records to a pandas DataFrame, which is faster to operate on.
-    records = pd.DataFrame.from_records(self._headers)
+    records = pd.DataFrame.from_dict(self._headers)
     # Keep track of original dtypes (may need to re-cast later).
-    original_dtypes = dict(self._headers_dtype)
+    original_dtypes = dict([(key,value.dtype) for key,value in self._headers.items()])
 
     # Ignore deleted / invalidated records.
     records = records[records['dltf']==0]
@@ -847,10 +878,9 @@ class BufferBase (object):
   # Choose which method to iterate over the data
   # (depending on if pandas is installed).
   def _makevars (self):
-    try:
-      import pandas as pd
+    if _use_pandas():
       self._makevars_pandas()
-    except ImportError:
+    else:
       self._makevars_slow()
 
   # Iterate over all unique axes found in the variables.
@@ -901,7 +931,7 @@ class BufferBase (object):
     if id(obj) in handled:
       return
 
-    if isinstance(obj,(_iter_type,_var_type,_axis_type,_dim_type)):
+    if isinstance(obj,(_iter_type,_chunk_type,_var_type,_axis_type,_dim_type)):
       yield obj
       handled.add(id(obj))
 
@@ -952,6 +982,15 @@ class BufferBase (object):
 
       return fstluk (key, dtype, rank, dataArray)
 
+  # How to decode the data from a raw binary array.
+  def _decode (self, data):
+    from fstd2nc.extra import decode
+    nbits = int(data[0x0b])
+    datyp = int(data[0x13])
+    dtype = dtype_fst2numpy(datyp, nbits)
+    out = decode(data).view(dtype)
+    return out
+
   #
   ###############################################
 
