@@ -72,8 +72,6 @@ class FSTD_Compat (BufferBase):
     import numpy as np
     import os
     import rpnpy.librmn.all as rmn
-    from fstd2nc.extra import librmn, file_table, stdf_dir_keys
-    from ctypes import cast, POINTER
 
     # This only works with an uncompressed netCDF4 file.
     nc_format = 'NETCDF4'
@@ -192,11 +190,15 @@ class FSTD_Compat (BufferBase):
         continue
       # Hard case: only have the record indices, need to loop over the records.
       # Get the shape of a single record for the variable.
-      if not hasattr(var,'record_id'): continue
-      record_shape = var.shape[var.record_id.ndim:]
+      if hasattr(var,'record_id'):
+        record_shape = var.shape[var.record_id.ndim:]
+      elif hasattr(var,'chunksize'):
+        record_shape = var.chunksize
+      else:
+        continue
       # Use this as the "chunk size" for the netCDF file, to improve I/O
       # performance.
-      chunksizes = (1,)*var.record_id.ndim + record_shape
+      chunksizes = (1,)*(len(var.axes)-len(record_shape)) + record_shape
       if hasattr(self,'_fill_value') and var.dtype.name.startswith('float32'):
         fill_value = self._fill_value
       else:
@@ -216,10 +218,13 @@ class FSTD_Compat (BufferBase):
       # Write the metadata.
       v.setncatts(var.atts)
       # Write the data.
-      indices = list(np.ndindex(var.record_id.shape))
-      # Sort the indices by FSTD key, so we're reading the records in the same
-      # order as they're found on disk.
-      keys = map(int,var.record_id.flatten())
+      if hasattr(var,'record_id'):
+        indices = list(np.ndindex(var.record_id.shape))
+        keys = map(int,var.record_id.flatten())
+      else:
+        indices = list(var.keys())
+        keys = list(var.chunks.values())
+        record_shape = None  # Reshaping with chunked data not supported.
       for r, ind in zip(keys,indices):
         if r >= 0:
           io.append((r,record_shape,v,ind))
@@ -238,7 +243,11 @@ class FSTD_Compat (BufferBase):
     for r,shape,v,ind in bar.iter(sorted(io)):
       try:
         data = self._fstluk(r,dtype=v.dtype.newbyteorder('='))['d'].transpose().reshape(shape)
-        chunk_addresses[r] = write_data (v, ind, data)
+        addr = write_data (v, ind, data)
+        if addr is not None:
+          chunk_addresses[r] = addr
+        else:
+          warn(_("Problem writing compatible record for %s:%s.  Writing separate netCDF / FSTD versions instead.")%(v.name,ind))
       except (IndexError,ValueError):
         warn(_("Internal problem with the script - unable to get data for '%s'")%v.name)
         continue
@@ -285,8 +294,6 @@ class FSTD_Compat (BufferBase):
 
     # Write anything which isn't already in the file.
     iun = rmn.fstopenall(filename,rmn.FST_RW)
-    index = librmn.file_index(iun)
-    f = file_table[index].contents
     for rec_id in unwritten_rec_ids:
       d = self._fstluk(rec_id)
       # This data might have already been written as a netCDF coordinate array.
@@ -301,11 +308,9 @@ class FSTD_Compat (BufferBase):
       # This data wasn't used for the netCDF4 interface, so don't have to worry
       # about making it netCDF-compatible.
       rmn.fstecr(iun,d)
-      # Keep track of where the data was written.  This information is
-      # available in the low-level librmn structures.
-      p = f.cur_dir_page.contents.dir
-      entry = cast(p.entry,POINTER(stdf_dir_keys))[p.nent-1]
-      chunk_addresses[rec_id] = (entry.addr-1)*8+72, (entry.lng*8)-96, d['datyp'], d['nbits']
+      # Keep track of where the data was written.
+      prm = rmn.fstprm(rmn.fstinl(iun)[-1])
+      chunk_addresses[rec_id] = (prm['swa']-1)*8+72, (prm['lng']*4)-96, d['datyp'], d['nbits']
     rmn.fstcloseall(iun)
 
     # By this point, all data is in the file.  This includes netCDF-only data,
@@ -325,7 +330,9 @@ class FSTD_Compat (BufferBase):
     datyps = np.asarray(datyps)
     nbits = np.asarray(nbits)
     # Encode the record headers.
-    headers = self._headers[list(rec_ids)]
+    from fstd2nc.extra import structured_array
+    headers = structured_array(self._headers)
+    headers = headers[list(rec_ids)]
     nrecs = len(headers)
     buf = np.zeros((nrecs,18),'>i4')
     # deleted, select, size
@@ -407,6 +414,10 @@ class FSTD_Compat (BufferBase):
           checksum ^= b
         np.array([checksum,0],'>i4').tofile(f)  # checksum, reserved
         buf[rec0:rec0+256].tofile(f)
+        # Pad last page to 256 entries.
+        nrecs_written = buf[rec0:rec0+256].shape[0]
+        if nrecs_written < 256:
+          np.zeros((256-nrecs_written,18),'>i4').tofile(f)
 
         if rec0 != 0:
           # Update total number of pages.
