@@ -25,147 +25,146 @@ import collections
 #################################################
 # Provide various external array interfaces for the FSTD data.
 
-# Helper function to embed information about the preferred chunk order.
-# Use as a wrapper when constructing dask Array objects.
-def _preferred_chunk_order (group, index, array):
-  return array
+# Method for reading a block from a file.
+def _read_block (filename, offset, length):
+  import numpy as np
+  with open(filename,'rb') as f:
+    f.seek (offset,0)
+    return np.fromfile(f,'B',length)
 
-# Helper interface for ordering dask tasks based on FSTD record order.
-# Might provide a small speed boost when the OS employs a read-ahead buffer.
-# Based on dask.core.get_dependencies
-class _RecordOrder (object):
-  def __init__(self, dsk):
-    self.dask = dsk
-  def __call__ (self, arg):
-    dsk = self.dask
-    work = [arg]
-    while work:
-        new_work = []
-        for w in work:
-            typ = type(w)
-            if typ is tuple and w and isinstance(w[0], collections.Callable):  # istask(w)
-                if w[0] is _preferred_chunk_order:
-                  return w[1], w[2]
-                else:
-                  new_work += w[1:]
-            elif typ is list:
-                new_work += w
-            elif typ is dict:
-                new_work += list(w.values())
-            else:
-                try:
-                    if w in dsk:
-                        new_work.append(dsk[w])
-                except TypeError:  # not hashable
-                    pass
-        work = new_work
+# Method for selecting a subset of data from a block of data.
+def _subset (block, start, end):
+  return block[start:end]
 
-# Add a callback to dask to ensure FSTD records are read in a good order.
-try:
-  from dask.callbacks import Callback
-  class _FSTD_Callback (Callback):
-    def _start_state (self, dsk, state):
-      ready = sorted(state['ready'][::-1],key=_RecordOrder(dsk))
-      state['ready'] = ready[::-1]
-  _FSTD_Callback().register()
-  del Callback
-
-except ImportError:
-  pass
+# Method for merging arrays together.
+def _concat (*data):
+  import numpy as np
+  return np.concatenate(data)
 
 
 class ExternOutput (BufferBase):
 
-  _read_chunk_cache = None
-
-  # The interface for getting chunks into dask.
-  def _read_chunk (self, rec_id, shape, dtype):
-    with self._lock:
-      # Cache the last record read from this interface, in case it is
-      # immediately requested again.
-      # Can happen, for instance, if the same data is sliced in multiple
-      # different ways.
-      if self._read_chunk_cache is None:
-        self._read_chunk_cache = {}
-      if rec_id not in self._read_chunk_cache:
-        self._read_chunk_cache.clear()
-        self._read_chunk_cache[rec_id] = self._fstluk(rec_id,dtype=dtype)['d'].transpose().reshape(shape)
-      return self._read_chunk_cache[rec_id]
+  # Helper method to get graph for raw input data.
+  def _graphs (self):
+    from fstd2nc.extra import blocksize
+    from os.path import getsize
+    import numpy as np
+    graphs = [None] * self._nrecs
+    ###
+    # Special case: already have a dask object from external source.
+    # (E.g., from fstpy)
+    if hasattr(self, '_extern_table'):
+      for rec_id in range(self._nrecs):
+        d = self._extern_table['d'].iloc[rec_id]
+        if hasattr(d,'compute'):
+          # Start a dask graph using the external dask array as the source.
+          graph = (d.compute,)
+          graph = (np.ravel, graph, 'K')
+        else:  # Special case: have a numpy array in memory.
+          graph = d
+          graph = np.ravel(graph, 'K')
+        graphs[rec_id] = {None:graph}
+      return graphs
+    ###
+    # Otherwise, construct graphs with our own dask wrapper.
+    blocksizes = dict()
+    fs_blocks = dict()
+    all_file_ids = np.array(self._headers['file_id'],copy=True)
+    all_swa = np.array(self._headers['swa'],copy=True)
+    all_lng = np.array(self._headers['lng'],copy=True)
+    for rec_id in range(self._nrecs):
+      graph = dict()
+      file_id = all_file_ids[rec_id]
+      filename = self._files[file_id]
+      if filename not in blocksizes:
+        blocksizes[filename] = max(blocksize(filename), 2**20)
+      bs = blocksizes[filename]
+      offset = all_swa[rec_id] * 8 - 8
+      length = all_lng[rec_id] * 4
+      current_block = offset // bs
+      pieces = []
+      while current_block * bs < offset + length:
+        s1 = max(current_block*bs, offset) - current_block*bs
+        s2 = min(current_block*bs+bs, offset+length) - current_block*bs
+        if (filename,current_block) not in fs_blocks:
+          fs_blocks[(filename,current_block)] = (_read_block, filename, current_block*bs, bs)
+        graph[filename+'-block-'+str(current_block)] = fs_blocks[(filename,current_block)]
+        pieces.append( (_subset, filename+'-block-'+str(current_block), s1, s2) )
+        current_block = current_block + 1
+      if len(pieces) > 1:
+        graph[filename+'-raw-'+str(offset)] = (_concat,) + tuple(pieces)
+      else:
+        graph[filename+'-raw-'+str(offset)] = pieces[0]
+      graph[filename+'-decode-'+str(offset)] = (np.transpose, (self._decode, filename+'-raw-'+str(offset)))
+      graph[None] = filename+'-decode-'+str(offset)
+      graphs[rec_id] = graph
+    return graphs
 
   def _iter_dask (self):
     """
     Iterate over all the variables, and convert to dask arrays.
     """
-    from fstd2nc.mixins import _iter_type, _var_type
+    from fstd2nc.mixins import _iter_type, _chunk_type, _var_type
     from dask import array as da
     from dask.base import tokenize
     import numpy as np
     from itertools import product
+    from dask import delayed
     unique_token = tokenize(self._files,id(self))
-    # Make a local copy of two columns from the header table.
-    # This speeds up access to their elements in the inner loop.
-    all_file_ids = np.array(self._headers['file_id'],copy=True)
-    all_keys = np.array(self._headers['key'],copy=True)
-    ###
-    # For science network installation, make sure the stack size of child
-    # threads is large enough to accomodate workspace for librmn.
-    from os.path import basename, splitext
-    from rpnpy.librmn import librmn
-    libname = basename(getattr(librmn,'_name',''))
-    # Skip this step for patched versions of librmn (ending in -rpnpy) that
-    # use the heap instead of the stack for workspace.
-    if not splitext(libname)[0].endswith('-rpnpy'):
-      import threading
-      # Use the size of the largest record, plus 1MB just in case.
-      # Use multiples of 4K for the size, as suggested in Python theading
-      # documentation.
-      # https://docs.python.org/3/library/threading.html#threading.stack_size
-      # Allow for possibility of double precision values (64-bit)
-      arraysize = np.max(8 * self._headers['ni'] * self._headers['nj'] * self._headers['nk'])
-      stacksize = arraysize//4096*4096 + (1<<20)
-      if stacksize > threading.stack_size():
-        threading.stack_size(stacksize)
-    ###
+    graphs = self._graphs()
     self._makevars()
     for var in self._iter_objects():
+      if not isinstance(var,(_iter_type,_chunk_type)):
+        yield var
+        continue
+      name = var.name+"-"+unique_token
+      ndim = len(var.axes)
+      shape = var.shape
+      # Convert _iter_type to more generic _chunk_type.
       if isinstance(var,_iter_type):
-        name = var.name+"-"+unique_token
-        ndim = len(var.axes)
-        shape = var.shape
+        chunks = {}
         ndim_outer = var.record_id.ndim
         ndim_inner = ndim - ndim_outer
-        inner_zeros = (0,)*ndim_inner
-        dsk = dict()
-        chunk_shape = (1,)*ndim_outer+shape[ndim_outer:]
+        chunk_shape = shape[ndim_outer:]
         for ind in product(*map(range,var.record_id.shape)):
-          # Pad index with all dimensions (including inner ones).
-          key = (name,) + ind + inner_zeros
-          rec_id = int(var.record_id[ind])
+          rec_id = var.record_id[ind]
+          ind = ind + tuple((0,dx) for dx in shape[ndim_outer:])
+          chunks[ind] = rec_id
+        var = _chunk_type (var.name, var.atts, var.axes, var.dtype, chunks, chunk_shape)
+      # Convert _chunk_type to dask Array objects.
+      if isinstance(var,_chunk_type):
+        ndim_inner = len(var.chunksize)
+        ndim_outer = ndim - ndim_inner
+        # Get chunk dimensions.
+        # First, size of single (untruncated) chunk, full indices.
+        untruncated_chunksize = (1,)*(ndim-len(var.chunksize)) + var.chunksize
+        # Next, breakdown of chunks along all variable dimensions.
+        chunks = []
+        chunk_indices = []
+        for i in range(ndim):
+          dx = untruncated_chunksize[i]
+          ch = tuple(dx for j in range(dx,shape[i]+1,dx))
+          if shape[i] % dx > 0:
+            ch = ch + (shape[i] % dx, )
+          chunks.append(ch)
+          chunk_indices.append(range(len(ch)))
+        # Loop over all indices, generate dask graph.
+        dsk = dict()
+        for ind, chunk_shape in zip(product(*chunk_indices), product(*chunks)):
+          # Unique key for this graph member.
+          key = (name,) + ind
+          # Get record id.
+          slices = [(i*dx,i*dx+res) for i,dx,res in zip(ind,untruncated_chunksize,chunk_shape)]
+          slices[:ndim_outer] = ind[:ndim_outer]
+          rec_id = var.chunks.get(tuple(slices),-1)
           # Add this record as a chunk in the dask Array.
           # Also, specify the preferred order of reading the chunks within the
           # file.
           if rec_id >= 0:
-            file_id = all_file_ids[rec_id]
-            filename = self._files[file_id]
-            rec_key = all_keys[rec_id]
-            # Special case: already have a dask object from external source.
-            # (E.g., from fstpy)
-            if hasattr(self, '_extern_table'):
-              d = self._extern_table['d'].iloc[rec_id]
-              if hasattr(d,'compute'):
-                # Start a dask graph using the external dask array as the source.
-                dsk[key] = (d.compute,)
-                # Ensure the dask array includes the degenerate outer dimensions.
-                # Otherwise get a runtime error if slicing is done on this.
-                dsk[key] = (np.ravel, dsk[key], 'K')
-                dsk[key] = (np.reshape, dsk[key], chunk_shape)
-              else:  # Special case: have a numpy array in memory.
-                dsk[key] = d
-                dsk[key] = np.ravel(dsk[key], 'K')
-                dsk[key] = np.reshape(dsk[key], chunk_shape)
-            # Otherwise, construct one with our own dask wrapper.
-            else:
-              dsk[key] = (_preferred_chunk_order,filename,rec_key,(self._read_chunk, rec_id, chunk_shape, var.dtype))
+            graph = graphs[rec_id][None]
+            dsk.update(graphs[rec_id])
+            del dsk[None]
+            dsk[key] = (np.reshape, graph, chunk_shape)
           else:
             # Fill missing chunks with fill value or NaN.
             if hasattr(self,'_fill_value'):
@@ -173,11 +172,6 @@ class ExternOutput (BufferBase):
               dsk[key] = (np.full, chunk_shape, self._fill_value, var.dtype)
             else:
               dsk[key] = (np.full, chunk_shape, float('nan'), var.dtype)
-        chunks = []
-        for i in range(ndim_outer):
-          chunks.append((1,)*shape[i])
-        for i in range(ndim_outer,ndim):
-          chunks.append((shape[i],))
         array = da.Array(dsk, name, chunks, var.dtype)
         var = _var_type(var.name,var.atts,var.axes,array)
       yield var
@@ -236,6 +230,7 @@ class ExternOutput (BufferBase):
     Create a pygeode interface for the RPN data.
     Requires pygeode >= 1.2.0, and xarray/dask.
     """
+    _fix_to_pygeode()
     from pygeode.ext_xarray import from_xarray
     data = self.to_xarray()
     return from_xarray(data)
@@ -286,16 +281,18 @@ class ExternOutput (BufferBase):
 # Workaround for recent xarray (>0.10.0) which changed the methods in the
 # conventions module.
 # Fixes an AttributeError when using to_pygeode().
-try:
-  from xarray.coding import times
-  from xarray import conventions
-  if not hasattr(conventions,'maybe_encode_datetime'):
-    conventions.maybe_encode_datetime = times.CFDatetimeCoder().encode
-  if not hasattr(conventions,'maybe_encode_timedelta'):
-    conventions.maybe_encode_timedelta = times.CFTimedeltaCoder().encode
-except (ImportError,AttributeError):
-  pass
-
+def _fix_to_pygeode (fixed=[False]):
+  if fixed[0] is True: return
+  try:
+    from xarray.coding import times
+    from xarray import conventions
+    if not hasattr(conventions,'maybe_encode_datetime'):
+      conventions.maybe_encode_datetime = times.CFDatetimeCoder().encode
+    if not hasattr(conventions,'maybe_encode_timedelta'):
+      conventions.maybe_encode_timedelta = times.CFTimedeltaCoder().encode
+  except (ImportError,AttributeError):
+    pass
+  fixed[0] = True
 
 class ExternInput (BufferBase):
   @classmethod
@@ -308,13 +305,20 @@ class ExternInput (BufferBase):
       table = table.to_pandas()
     # Construct the record header info from the table.
     fields = ['nomvar', 'typvar', 'etiket', 'ni', 'nj', 'nk', 'dateo', 'ip1', 'ip2', 'ip3', 'deet', 'npas', 'datyp', 'nbits', 'grtyp', 'ig1', 'ig2', 'ig3', 'ig4', 'datev']
-    headers = np.zeros(len(table), dtype=cls.__new__(cls,**kwargs)._headers_dtype)
+    headers = {}
     for col in fields:
-      headers[col][:] = table[col]
+      headers[col] = table[col].values.copy()
     # Pad out string variables with spaces.
+    headers['nomvar'] = np.asarray(headers['nomvar'], dtype='|S4')
+    headers['typvar'] = np.asarray(headers['nomvar'], dtype='|S2')
+    headers['etiket'] = np.asarray(headers['etiket'], dtype='|S12')
+    headers['grtyp'] = np.asarray(headers['grtyp'], dtype='|S1')
     headers['nomvar'] = np.char.ljust(headers['nomvar'], 4, ' ')
     headers['typvar'] = np.char.ljust(headers['typvar'], 2, ' ')
     headers['etiket'] = np.char.ljust(headers['etiket'], 12, ' ')
+    # Add other fields that may be needed.
+    if 'dltf' not in headers:
+      headers['dltf'] = np.zeros(len(headers['nomvar']), dtype='int32')
     # Generate temporary file with target grid info.
     try: # Python 3
       grid_tmpdir = tempfile.TemporaryDirectory()
@@ -325,15 +329,15 @@ class ExternInput (BufferBase):
     # Write all grid records to a temporary file, so they are accessible to
     # the vgrid / librmn helper functions.
     iun = rmn.fstopenall(gridfile, rmn.FST_RW)
-    for nomvar in ('!!','>>','^^','^>','!!5F'):
-      for ind in np.where(table['nomvar'] == nomvar)[0]:
+    for nomvar in (b'!!  ',b'>>  ',b'^^  ',b'^>  ',b'!!SF'):
+      for ind in np.where(headers['nomvar'] == nomvar)[0]:
         rec = table.iloc[ind].to_dict()
         rec['d'] = np.asarray(rec['d'])
         rmn.fstecr(iun, rec)
     rmn.fstcloseall(iun)
 
     # Initialize the Buffer object with this info.
-    b = cls(gridfile, header_cache={'__ROOT__'+gridfile:headers}, **kwargs)
+    b = cls(gridfile, _headers=headers, **kwargs)
     b._grid_tmpdir = grid_tmpdir  # Save tmpdir until cleanup.
 
     # Save the dataframe for reference.
