@@ -21,6 +21,23 @@
 from fstd2nc.stdout import _, info, warn, error
 from fstd2nc.mixins import BufferBase
 
+# Helper function - apply metadata and renames to the variables.
+def apply_metadata (b):
+  if hasattr(b,'_metadata'):
+    # Apply the user-supplied metadata.
+    for obj in b._iter_objects():
+      # Add extra metadata provided by the user?
+      if obj.name in b._metadata:
+        for attname, attval in b._metadata[obj.name].items():
+          # Check if attribute should be removed?
+          # (i.e., if value is intentionally left empty)
+          if attval == "": obj.atts.pop(attname,None)
+          else: obj.atts[attname] = attval
+      # Apply renames.
+      if obj.name in b._renames:
+        obj.name = b._renames[obj.name]
+
+
 #################################################
 # Mixin for adding netCDF metadata to the variables
 
@@ -119,23 +136,13 @@ class netCDF_Atts (BufferBase):
     super(netCDF_Atts,self)._makevars()
 
     # Extract variable rename requests from the user-supplied metadata.
-    renames = {}
+    self._renames = {}
     for varname, atts in self._metadata.items():
       if 'rename' in atts:
-        renames[varname] = atts.pop('rename')
+        self._renames[varname] = atts.pop('rename')
 
     # Apply the user-supplied metadata.
-    for obj in self._iter_objects():
-      # Add extra metadata provided by the user?
-      if obj.name in self._metadata:
-        for attname, attval in self._metadata[obj.name].items():
-          # Check if attribute should be removed?
-          # (i.e., if value is intentionally left empty)
-          if attval == "": obj.atts.pop(attname,None)
-          else: obj.atts[attname] = attval
-      # Apply renames.
-      if obj.name in renames:
-        obj.name = renames[obj.name]
+    apply_metadata (self)
 
 
 
@@ -188,8 +195,11 @@ class netCDF_IO (BufferBase):
       if var.dims.index('time') > 0:
         self._time_unlimited = False
 
+    # Generate unique names for output data.
     if self._unique_names:
       self._fix_names()
+      # Hack: re-apply metadata for the final (renamed) variables.
+      apply_metadata(self)
 
     for var in self._iter_objects():
       # Modify time axes to be relative units instead of datetime objects.
@@ -250,7 +260,7 @@ class netCDF_IO (BufferBase):
   def _fix_names (self):
 
     # List of metadata keys that are internal to the FSTD file.
-    internal_meta = self._headers.dtype.names
+    internal_meta = self._headers.keys()
 
     # Generate unique axis names.
     axis_table = dict()
@@ -381,11 +391,15 @@ class netCDF_IO (BufferBase):
         continue
       # Hard case: only have the record indices, need to loop over the records.
       # Get the shape of a single record for the variable.
-      if not hasattr(var,'record_id'): continue
-      record_shape = var.shape[var.record_id.ndim:]
+      if hasattr(var,'record_id'):
+        record_shape = var.shape[var.record_id.ndim:]
+      elif hasattr(var,'chunksize'):
+        record_shape = var.chunksize
+      else:
+        continue
       # Use this as the "chunk size" for the netCDF file, to improve I/O
       # performance.
-      chunksizes = (1,)*var.record_id.ndim + record_shape
+      chunksizes = (1,)*(len(var.axes)-len(record_shape)) + record_shape
       if hasattr(self,'_fill_value') and var.dtype.name.startswith('float32'):
         fill_value = self._fill_value
       else:
@@ -403,10 +417,13 @@ class netCDF_IO (BufferBase):
       # Write the metadata.
       v.setncatts(var.atts)
       # Write the data.
-      indices = list(np.ndindex(var.record_id.shape))
-      # Sort the indices by FSTD key, so we're reading the records in the same
-      # order as they're found on disk.
-      keys = map(int,var.record_id.flatten())
+      if hasattr(var,'record_id'):
+        indices = list(np.ndindex(var.record_id.shape))
+        keys = map(int,var.record_id.flatten())
+      else:
+        indices = list(var.keys())
+        keys = list(var.chunks.values())
+        record_shape = None  # Reshaping with chunked data not supported.
       for r, ind in zip(keys,indices):
         if r >= 0:
           io.append((r,record_shape,v,ind))
