@@ -56,12 +56,6 @@ class VCoords (BufferBase):
     parser.add_argument('--momentum-levels', '--mlev', action='store_true', help=_("Only convert data that's on 'momentum' vertical levels."))
     parser.add_argument('--vertical-velocity-levels', '--wlev', action='store_true', help=_("Only convert data that's on 'vertical velocity' levels."))
 
-  # Need to extend _headers_dtype before __init__.
-  def __new__ (cls, *args, **kwargs):
-    obj = super(VCoords,cls).__new__(cls, *args, **kwargs)
-    obj._headers_dtype = obj._headers_dtype + [('level','float32'),('kind','int32'),('level_descr','|S20'),('special_level','|S20')]
-    return obj
-
   def __init__ (self, *args, **kwargs):
     """
     strict_vcoord_match : bool, optional
@@ -231,6 +225,7 @@ class VCoords (BufferBase):
 
     # Apply special labels for some levels.
     # Used to describe a variable when it's split into multiple coordinate types.
+    fields['level_descr'] = np.empty(len(fields['kind']), dtype='|S20')
     fields['level_descr'][:] = 'unknown_levels'
     fields['level_descr'][fields['kind'] == 0] = 'depth_levels'
     fields['level_descr'][fields['kind'] == 1] = 'model_levels'
@@ -239,6 +234,7 @@ class VCoords (BufferBase):
     fields['level_descr'][fields['kind'] == 4] = 'height_levels'
     fields['level_descr'][fields['kind'] == 5] = 'model_levels'
     fields['level_descr'][fields['kind'] == 21] = 'model_levels'
+    fields['special_level'] = np.empty(len(fields['kind']), dtype='|S20')
     fields['special_level'][:] = fields['level_descr'][:]
     # Discern ocean depths from ocean bottom.
     fields['special_level'][fields['kind'] == 0] = 'depth_levels'
@@ -280,8 +276,8 @@ class VCoords (BufferBase):
     for var in self._varlist:
       level_axis = var.getaxis('level')
       if level_axis is None: continue
-      # Degenerate vertical axis?
-      if 'ip1' in var.atts and var.atts['ip1'] == 0:
+      # Degenerate vertical axis (0mb or 0hy?)
+      if 'ip1' in var.atts and var.atts['ip1'] in (0, 99614720):
         if len(level_axis) == 1:
           i = var.dims.index('level')
           var.axes.pop(i)
@@ -418,6 +414,9 @@ class VCoords (BufferBase):
               coordC.array = np.asarray(C)
             except (KeyError,ValueError,VGDError):
               warn (_("Unable to get A/B coefficients for %s.")%var.name)
+              coordA = None
+              coordB = None
+              coordC = None
             vgd_free (vgd_id)
         ###
 
@@ -480,11 +479,11 @@ class VCoords (BufferBase):
                 atts['standard_name'] = 'atmosphere_hybrid_sigma_ln_pressure_coordinate'
                 # Document the formula to follow, since it's not in the conventions.
                 #TODO: update this once there's an actual convention to follow!
-                if version in (1,2,3,4,5):
+                if version in (1,2,3,4,5) and None not in (coordA,coordB):
                   atts['formula'] = "p = exp(a+b*log(ps/pref)) / 100.0"
                   atts['formula_terms'] = OrderedDict([('a',coordA),('b',coordB),('ps','P0'),('pref','pref')])
                   coordinates.extend([coordA,coordB])
-                elif sleve:
+                elif sleve and None not in (coordA,coordB,coordC):
                   atts['formula'] = "p = exp(a+b*log(ps/pref)+c*log(psl/pref)) / 100.0"
                   atts['formula_terms'] = OrderedDict([('a',coordA),('b',coordB),('c',coordC),('ps','P0'),('psl','P0LS'),('pref','pref')])
                   coordinates.extend([coordA,coordB,coordC])
@@ -498,11 +497,12 @@ class VCoords (BufferBase):
                   pass # Don't have PREF available for some reason?
               else:
                 atts['standard_name'] = 'atmosphere_hybrid_sigma_pressure_coordinate'
-                coordA.array /= 100.0  # Use hPa for final units.
-                atts['formula'] = 'p = ap + b*ps'
-                coordA.name = 'ap'
-                atts['formula_terms'] = OrderedDict([('ap',coordA),('b',coordB),('ps','P0')])
-                coordinates.extend([coordA,coordB])
+                if None not in (coordA,coordB):
+                  coordA.array /= 100.0  # Use hPa for final units.
+                  atts['formula'] = 'p = ap + b*ps'
+                  coordA.name = 'ap'
+                  atts['formula_terms'] = OrderedDict([('ap',coordA),('b',coordB),('ps','P0')])
+                  coordinates.extend([coordA,coordB])
 
             # Not a '!!' coordinate, so must be 'HY'?
             else:
@@ -539,19 +539,21 @@ class VCoords (BufferBase):
           if sleve:
             atts['standard_name'] = 'atmosphere_sleve_coordinate'
             atts['positive'] = 'up'
-            atts['formula'] = 'z = az + b1*zsurf1 + b2*zsurf2'
-            coordA.name = 'az'
-            coordB.name = 'b2'
-            coordC.name = 'b1'
-            atts['formula_terms'] = OrderedDict([('az',coordA),('b1',coordC),('zsurf1','MELS'),('b2',coordB),('zsurf2','ME')])
-            coordinates.extend([coordA,coordB,coordC])
+            if None not in (coordA,coordB,coordC):
+              atts['formula'] = 'z = az + b1*zsurf1 + b2*zsurf2'
+              coordA.name = 'az'
+              coordB.name = 'b2'
+              coordC.name = 'b1'
+              atts['formula_terms'] = OrderedDict([('az',coordA),('b1',coordC),('zsurf1','MELS'),('b2',coordB),('zsurf2','ME')])
+              coordinates.extend([coordA,coordB,coordC])
 
           else:
             atts['standard_name'] = 'atmosphere_hybrid_height_coordinate'
             atts['positive'] = 'up'
-            atts['formula'] = 'z = a + b*orog'
-            atts['formula_terms'] = OrderedDict([('a',coordA),('b',coordB),('orog','ME')])
-            coordinates.extend([coordA,coordB])
+            if None not in (coordA,coordB):
+              atts['formula'] = 'z = a + b*orog'
+              atts['formula_terms'] = OrderedDict([('a',coordA),('b',coordB),('orog','ME')])
+              coordinates.extend([coordA,coordB])
 
         # Add this vertical axis.
         if len(coordinates) > 0:
